@@ -21,6 +21,7 @@
 #ifndef DATA_H
 #define DATA_H
 
+#include <algorithm>
 #include <QString>
 #include <QList>
 
@@ -33,6 +34,14 @@
  * @author Percy Leonhardt <percy@eris23.de>
  */
 
+using namespace std;
+
+/* This is for clamping min/max values read from the settings file */
+template <class T> inline T clamp(T x, T a, T b)
+{
+	return min(max(x,a),b);
+}
+
 struct InterfaceCommand
 {
     bool runAsRoot;
@@ -43,8 +52,7 @@ struct InterfaceCommand
 struct InterfaceSettings
 {
     InterfaceSettings()
-      : iconSet( 0 ),
-        numCommands( 0 ),
+      : numCommands( 0 ),
         trafficThreshold( 0 ),
         hideWhenNotExisting( false ),
         hideWhenNotAvailable( false ),
@@ -52,7 +60,7 @@ struct InterfaceSettings
         customCommands( false )
     {}
 
-    int iconSet;
+    QString iconSet;
     int numCommands;
     int trafficThreshold;
     bool hideWhenNotExisting;
