@@ -51,14 +51,11 @@
 #include "configdialog.h"
 #include "../knemod/backends/kcmregistry.h"
 
-const QString ConfigDialog::ICON_DISCONNECTED = "network_disconnected";
-const QString ConfigDialog::ICON_CONNECTED = "network_connected";
-const QString ConfigDialog::ICON_INCOMING = "network_incoming";
-const QString ConfigDialog::ICON_OUTGOING = "network_outgoing";
-const QString ConfigDialog::ICON_TRAFFIC = "network_traffic";
-const QString ConfigDialog::SUFFIX_PPP = "_ppp";
-const QString ConfigDialog::SUFFIX_LAN = "_lan";
-const QString ConfigDialog::SUFFIX_WLAN = "_wlan";
+const QString ConfigDialog::ICON_DISCONNECTED = "_disconnected";
+const QString ConfigDialog::ICON_CONNECTED = "_connected";
+const QString ConfigDialog::ICON_INCOMING = "_incoming";
+const QString ConfigDialog::ICON_OUTGOING = "_outgoing";
+const QString ConfigDialog::ICON_TRAFFIC = "_traffic";
 
 
 K_PLUGIN_FACTORY(KNemoFactory, registerPlugin<ConfigDialog>();)
@@ -83,6 +80,23 @@ ConfigDialog::ConfigDialog( QWidget *parent, const QVariantList &args )
     mDlg->setupUi( main );
     top->addWidget( main );
 
+    // Search for iconsets and add them to comboBoxIconSet
+    KStandardDirs iconDirs;
+    iconDirs.addResourceType("knemo_pics", "data", "knemo/pics");
+    QStringList iconlist = iconDirs.findAllResources( "knemo_pics", "*.png" );
+
+    foreach ( QString iconName, iconlist )
+    {
+        QRegExp rx( "pics\\/(.+)_(connected|disconnected|incoming|outgoing|traffic)\\.png" );
+        if ( rx.indexIn( iconName ) > -1 )
+            if ( !mIconSets.contains( rx.cap( 1 ) ) )
+                mIconSets << rx.cap( 1 );
+    }
+    mIconSets.sort();
+    mDlg->comboBoxIconSet->addItems( mIconSets );
+    if ( mIconSets.contains( "monitor" ) )
+        mDlg->comboBoxIconSet->setCurrentIndex( mIconSets.indexOf( "monitor" ) );
+
     // fill the backends combobox
     QStringList items;
     for ( int i = 0; KCMRegistry[i].name != QString::null; i++ )
@@ -91,10 +105,11 @@ ConfigDialog::ConfigDialog( QWidget *parent, const QVariantList &args )
     }
     mDlg->comboBoxBackends->addItems( items );
 
-    mDlg->pushButtonNew->setIcon( SmallIcon( "document-new" ) );
-    mDlg->pushButtonDelete->setIcon( SmallIcon( "edit-delete" ) );
-    mDlg->pushButtonAddCommand->setIcon( SmallIcon( "document-new" ) );
-    mDlg->pushButtonRemoveCommand->setIcon( SmallIcon( "edit-delete" ) );
+    mDlg->pushButtonNew->setIcon( SmallIcon( "list-add" ) );
+    mDlg->pushButtonAll->setIcon( SmallIcon( "document-new" ) );
+    mDlg->pushButtonDelete->setIcon( SmallIcon( "list-remove" ) );
+    mDlg->pushButtonAddCommand->setIcon( SmallIcon( "list-add" ) );
+    mDlg->pushButtonRemoveCommand->setIcon( SmallIcon( "list-remove" ) );
     mDlg->pushButtonUp->setIcon( SmallIcon( "arrow-up" ) );
     mDlg->pushButtonDown->setIcon( SmallIcon( "arrow-down" ) );
     mDlg->pushButtonAddToolTip->setIcon( SmallIcon( "arrow-right" ) );
@@ -112,11 +127,12 @@ ConfigDialog::ConfigDialog( QWidget *parent, const QVariantList &args )
                           "the command as root user check the corresponding " \
                           "<b>Root</b> CheckBox.") );
 
-    //mSettingsDict.setAutoDelete( true );
     setButtons( KCModule::Default | KCModule::Apply );
 
     connect( mDlg->pushButtonNew, SIGNAL( clicked() ),
              this, SLOT( buttonNewSelected() ) );
+    connect( mDlg->pushButtonAll, SIGNAL( clicked() ),
+             this, SLOT( buttonAllSelected() ) );
     connect( mDlg->pushButtonDelete, SIGNAL( clicked() ),
              this, SLOT( buttonDeleteSelected() ) );
     connect( mDlg->pushButtonAddCommand, SIGNAL( clicked() ),
@@ -159,7 +175,7 @@ ConfigDialog::ConfigDialog( QWidget *parent, const QVariantList &args )
              this, SLOT( listViewCommandsChanged( QTreeWidgetItem*, int ) ) );
 
     // connect the plotter widgets
-    connect( mDlg->checkBoxTopBar, SIGNAL( toggled( bool ) ),
+    connect( mDlg->checkBoxBottomBar, SIGNAL( toggled( bool ) ),
              this, SLOT( checkBoxToggled( bool ) ) );
     connect( mDlg->checkBoxLabels, SIGNAL( toggled( bool ) ),
              this, SLOT( checkBoxToggled( bool ) ) );
@@ -175,8 +191,6 @@ ConfigDialog::ConfigDialog( QWidget *parent, const QVariantList &args )
              this, SLOT( checkBoxToggled( bool ) ) );
     connect( mDlg->checkBoxAutoDetection, SIGNAL( toggled( bool ) ),
              this, SLOT( checkBoxToggled( bool ) ) );
-    connect( mDlg->spinBoxCount, SIGNAL( valueChanged( int ) ),
-             this, SLOT( spinBoxValueChanged( int ) ) );
     connect( mDlg->spinBoxPixel, SIGNAL( valueChanged( int ) ),
              this, SLOT( spinBoxValueChanged( int ) ) );
     connect( mDlg->spinBoxDistance, SIGNAL( valueChanged( int ) ),
@@ -210,7 +224,7 @@ ConfigDialog::~ConfigDialog()
 
 void ConfigDialog::load()
 {
-    mSettingsDict.clear();
+    mSettingsMap.clear();
     mDlg->listBoxInterfaces->clear();
     KConfig *config = mConfig.data();
 
@@ -218,14 +232,14 @@ void ConfigDialog::load()
     bool startKNemo = generalGroup.readEntry( "AutoStart", true );
     mDlg->checkBoxStartKNemo->setChecked( startKNemo );
     mDlg->tabWidgetConfiguration->setEnabled( startKNemo );
-    mDlg->numInputPollInterval->setValue( generalGroup.readEntry( "PollInterval", 1 ) );
-    mDlg->numInputSaveInterval->setValue( generalGroup.readEntry( "SaveInterval", 60 ) );
+    mDlg->numInputPollInterval->setValue( clamp<int>(generalGroup.readEntry( "PollInterval", 1 ), 1, 60 ) );
+    mDlg->numInputSaveInterval->setValue( clamp<int>(generalGroup.readEntry( "SaveInterval", 60 ), 1, 300 ) );
     mDlg->lineEditStatisticsDir->setUrl( generalGroup.readEntry( "StatisticsDir", KGlobal::dirs()->saveLocation( "data", "knemo/" ) ) );
     mToolTipContent = generalGroup.readEntry( "ToolTipContent", 2 );
 
     // select the backend from the config file
     bool foundBackend = false;
-    QString backend = generalGroup.readEntry( "Backend", "Nettools" );
+    QString backend = generalGroup.readEntry( "Backend", "Sys" );
     int i;
     for ( i = 0; KCMRegistry[i].name != QString::null; i++ )
     {
@@ -238,20 +252,14 @@ void ConfigDialog::load()
 
     if ( !foundBackend )
     {
-        i = 0; // use the first backend (Nettools)
+        i = 0; // use the first backend (Sys)
     }
     mDlg->comboBoxBackends->setCurrentIndex( i );
     mDlg->textLabelBackendDescription->setText( KCMRegistry[i].description );
 
-    QStringList list;
-    list = generalGroup.readEntry( "Interfaces", list );
+    QStringList list = generalGroup.readEntry( "Interfaces", QStringList() );
 
-
-    if ( list.count() == 0 )
-        return;
-
-    QString interface;
-    foreach ( interface, list )
+    foreach ( QString interface, list )
     {
         QString group( "Interface_" );
         group += interface;
@@ -260,12 +268,12 @@ void ConfigDialog::load()
         {
             KConfigGroup interfaceGroup( config, group );
             settings->alias = interfaceGroup.readEntry( "Alias" );
-            settings->iconSet = interfaceGroup.readEntry( "IconSet", 0 );
+            settings->iconSet = interfaceGroup.readEntry( "IconSet", "monitor" );
             settings->customCommands = interfaceGroup.readEntry( "CustomCommands", false );
             settings->hideWhenNotAvailable = interfaceGroup.readEntry( "HideWhenNotAvailable", false );
             settings->hideWhenNotExisting = interfaceGroup.readEntry( "HideWhenNotExisting", false );
             settings->activateStatistics = interfaceGroup.readEntry( "ActivateStatistics", false );
-            settings->trafficThreshold = interfaceGroup.readEntry( "TrafficThreshold", 0 );
+            settings->trafficThreshold = clamp<int>(interfaceGroup.readEntry( "TrafficThreshold", 0 ), 0, 1000 );
             int numCommands = interfaceGroup.readEntry( "NumCommands", 0 );
             for ( int i = 0; i < numCommands; i++ )
             {
@@ -280,8 +288,11 @@ void ConfigDialog::load()
                 settings->commands.append( cmd );
             }
         }
-        mSettingsDict.insert( interface, settings );
+        mSettingsMap.insert( interface, settings );
         mDlg->listBoxInterfaces->addItem( interface );
+        mDlg->pushButtonDelete->setDisabled( false );
+        mDlg->groupBoxIfaceMisc->setDisabled( false );
+        mDlg->groupBoxIfaceMenu->setDisabled( false );
     }
 
     // enable or disable statistics entries
@@ -289,25 +300,24 @@ void ConfigDialog::load()
 
     // Set the plotter widgets
     KConfigGroup plotterGroup( config, "PlotterSettings" );
-    mDlg->spinBoxPixel->setValue( plotterGroup.readEntry( "Pixel", 1 ) );
-    mDlg->spinBoxCount->setValue( plotterGroup.readEntry( "Count", 5 ) );
-    mDlg->spinBoxDistance->setValue( plotterGroup.readEntry( "Distance", 30 ) );
-    mDlg->spinBoxFontSize->setValue( plotterGroup.readEntry( "FontSize", 8 ) );
-    mDlg->spinBoxMinValue->setValue( plotterGroup.readEntry( "MinimumValue", 0 ) );
-    mDlg->spinBoxMaxValue->setValue( plotterGroup.readEntry( "MaximumValue", 1 ) );
+    mDlg->spinBoxPixel->setValue( clamp<int>(plotterGroup.readEntry( "Pixel", 1 ), 1, 50 ) );
+    mDlg->spinBoxDistance->setValue( clamp<int>(plotterGroup.readEntry( "Distance", 30 ), 10, 120 ) );
+    mDlg->spinBoxFontSize->setValue( clamp<int>(plotterGroup.readEntry( "FontSize", 8 ), 5, 24 ) );
+    mDlg->spinBoxMinValue->setValue( clamp<int>(plotterGroup.readEntry( "MinimumValue", 0 ), 0, 49999 ) );
+    mDlg->spinBoxMaxValue->setValue( clamp<int>(plotterGroup.readEntry( "MaximumValue", 1 ), 1, 50000 ) );
     mDlg->checkBoxLabels->setChecked( plotterGroup.readEntry( "Labels", true ) );
-    mDlg->checkBoxTopBar->setChecked( plotterGroup.readEntry( "TopBar", false ) );
+    mDlg->checkBoxBottomBar->setChecked( plotterGroup.readEntry( "BottomBar", true ) );
     mDlg->checkBoxVLines->setChecked( plotterGroup.readEntry( "VerticalLines", true ) );
     mDlg->checkBoxHLines->setChecked( plotterGroup.readEntry( "HorizontalLines", true ) );
     mDlg->checkBoxIncoming->setChecked( plotterGroup.readEntry( "ShowIncoming", true ) );
     mDlg->checkBoxOutgoing->setChecked( plotterGroup.readEntry( "ShowOutgoing", true ) );
     mDlg->checkBoxAutoDetection->setChecked( plotterGroup.readEntry( "AutomaticDetection", true ) );
     mDlg->checkBoxVLinesScroll->setChecked( plotterGroup.readEntry( "VerticalLinesScroll", true ) );
-    mDlg->kColorButtonVLines->setColor( plotterGroup.readEntry( "ColorVLines", mColorVLines.rgb() ) );
-    mDlg->kColorButtonHLines->setColor( plotterGroup.readEntry( "ColorHLines", mColorHLines.rgb() ) );
-    mDlg->kColorButtonIncoming->setColor( plotterGroup.readEntry( "ColorIncoming", mColorIncoming.rgb() ) );
-    mDlg->kColorButtonOutgoing->setColor( plotterGroup.readEntry( "ColorOutgoing", mColorOutgoing.rgb() ) );
-    mDlg->kColorButtonBackground->setColor( plotterGroup.readEntry( "ColorBackground", mColorBackground.rgb() ) );
+    mDlg->kColorButtonVLines->setColor( plotterGroup.readEntry( "ColorVLines", mColorVLines ) );
+    mDlg->kColorButtonHLines->setColor( plotterGroup.readEntry( "ColorHLines", mColorHLines ) );
+    mDlg->kColorButtonIncoming->setColor( plotterGroup.readEntry( "ColorIncoming", mColorIncoming ) );
+    mDlg->kColorButtonOutgoing->setColor( plotterGroup.readEntry( "ColorOutgoing", mColorOutgoing ) );
+    mDlg->kColorButtonBackground->setColor( plotterGroup.readEntry( "ColorBackground", mColorBackground ) );
 
     // These things need to be here so that 'Reset' from the control
     // center is handled correctly.
@@ -368,31 +378,39 @@ void ConfigDialog::save()
     // Remove interfaces from the config that were deleted during this session
     foreach ( QString delIface, mDeletedIfaces )
     {
-        if ( !mSettingsDict.contains( delIface ) )
+        if ( !mSettingsMap.contains( delIface ) )
             config->deleteGroup( "Interface_" + delIface );
     }
 
-    foreach ( QString it, mSettingsDict.keys() )
+    foreach ( QString it, mSettingsMap.keys() )
     {
         list.append( it );
-        InterfaceSettings* settings = mSettingsDict.value( it );
+        InterfaceSettings* settings = mSettingsMap.value( it );
         KConfigGroup interfaceGroup( config, "Interface_" + it );
 
         // Preserve settings set by the app before delete
-        QByteArray plotterG, statisticsG, statusG;
-        plotterG = interfaceGroup.readEntry( "PlotterGeometry", plotterG );
-        statisticsG = interfaceGroup.readEntry( "StatisticsGeometry", statisticsG );
-        statusG = interfaceGroup.readEntry( "StatusGeometry", statusG );
+        QPoint plotterPos = interfaceGroup.readEntry( "PlotterPos", QPoint() );
+        QSize plotterSize = interfaceGroup.readEntry( "PlotterSize", QSize() );
+        QPoint statisticsPos = interfaceGroup.readEntry( "StatisticsPos", QPoint() );
+        QSize statisticsSize = interfaceGroup.readEntry( "StatisticsSize", QSize() );
+        QPoint statusPos = interfaceGroup.readEntry( "StatusPos", QPoint() );
+        QSize statusSize = interfaceGroup.readEntry( "StatusSize", QSize() );
 
         // Make sure we don't get crufty commands left over
         interfaceGroup.deleteGroup();
 
-        if ( !plotterG.isEmpty() )
-            interfaceGroup.writeEntry( "PlotterGeometry", plotterG );
-        if ( !statisticsG.isEmpty() )
-            interfaceGroup.writeEntry( "StatisticsGeometry", statisticsG );
-        if ( !statusG.isEmpty() )
-            interfaceGroup.writeEntry( "StatusGeometry", statusG );
+        if ( !plotterPos.isNull() )
+            interfaceGroup.writeEntry( "PlotterPos", plotterPos );
+        if ( !plotterSize.isEmpty() )
+            interfaceGroup.writeEntry( "PlotterSize", plotterSize );
+        if ( !statisticsPos.isNull() )
+            interfaceGroup.writeEntry( "StatisticsPos", statisticsPos );
+        if ( !statisticsSize.isEmpty() )
+            interfaceGroup.writeEntry( "StatisticsSize", statisticsSize );
+        if ( !statusPos.isNull() )
+            interfaceGroup.writeEntry( "StatusPos", statusPos );
+        if ( !statusSize.isEmpty() )
+            interfaceGroup.writeEntry( "StatusSize", statusSize );
         if ( !settings->alias.isEmpty() )
             interfaceGroup.writeEntry( "Alias", settings->alias );
 
@@ -416,23 +434,23 @@ void ConfigDialog::save()
     }
 
     KConfigGroup generalGroup( config, "General" );
+    generalGroup.writeEntry( "FirstStart", false );
     generalGroup.writeEntry( "AutoStart", mDlg->checkBoxStartKNemo->isChecked() );
     generalGroup.writeEntry( "PollInterval", mDlg->numInputPollInterval->value() );
     generalGroup.writeEntry( "SaveInterval", mDlg->numInputSaveInterval->value() );
-    generalGroup.writeEntry( "StatisticsDir",  mDlg->lineEditStatisticsDir->text() );
+    generalGroup.writeEntry( "StatisticsDir",  mDlg->lineEditStatisticsDir->url().url() );
     generalGroup.writeEntry( "Backend", mDlg->comboBoxBackends->currentText() );
     generalGroup.writeEntry( "ToolTipContent", mToolTipContent );
     generalGroup.writeEntry( "Interfaces", list );
 
     KConfigGroup plotterGroup( config, "PlotterSettings" );
     plotterGroup.writeEntry( "Pixel", mDlg->spinBoxPixel->value() );
-    plotterGroup.writeEntry( "Count", mDlg->spinBoxCount->value() );
     plotterGroup.writeEntry( "Distance", mDlg->spinBoxDistance->value() );
     plotterGroup.writeEntry( "FontSize", mDlg->spinBoxFontSize->value() );
     plotterGroup.writeEntry( "MinimumValue", mDlg->spinBoxMinValue->value() );
     plotterGroup.writeEntry( "MaximumValue", mDlg->spinBoxMaxValue->value() );
     plotterGroup.writeEntry( "Labels", mDlg->checkBoxLabels->isChecked() );
-    plotterGroup.writeEntry( "TopBar", mDlg->checkBoxTopBar->isChecked() );
+    plotterGroup.writeEntry( "BottomBar", mDlg->checkBoxBottomBar->isChecked() );
     plotterGroup.writeEntry( "VerticalLines", mDlg->checkBoxVLines->isChecked() );
     plotterGroup.writeEntry( "HorizontalLines", mDlg->checkBoxHLines->isChecked() );
     plotterGroup.writeEntry( "ShowIncoming", mDlg->checkBoxIncoming->isChecked() );
@@ -461,49 +479,62 @@ void ConfigDialog::save()
 
 void ConfigDialog::defaults()
 {
-    // Default interfaces
-    QFile proc( "/proc/net/dev" );
+    // Set these values before we check for default interfaces
+    mSettingsMap.clear();
+    mDlg->listBoxInterfaces->clear();
+    mDlg->pushButtonDelete->setDisabled( true );
+    mDlg->groupBoxIfaceMisc->setDisabled( true );
+    mDlg->groupBoxIfaceMenu->setDisabled( true );
+    mDlg->lineEditAlias->setText( QString::null );
+    mDlg->checkBoxNotConnected->setChecked( false );
+    mDlg->checkBoxNotExisting->setChecked( false );
+    mDlg->checkBoxStatistics->setChecked( false );
+    mDlg->checkBoxCustom->setChecked( false );
+    if ( mIconSets.contains( "monitor" ) )
+        mDlg->comboBoxIconSet->setCurrentIndex( mIconSets.indexOf( "monitor" ) );
+    else
+        mDlg->comboBoxIconSet->setCurrentIndex( 0 );
+    mDlg->pixmapDisconnected->clear();
+    mDlg->pixmapConnected->clear();
+    mDlg->pixmapIncoming->clear();
+    mDlg->pixmapOutgoing->clear();
+    mDlg->pixmapTraffic->clear();
+
+    // Default interface
+    QFile proc( "/proc/net/route" );
     if ( proc.open( QIODevice::ReadOnly ) )
     {
-        mSettingsDict.clear();
-        mDlg->listBoxInterfaces->clear();
-
         QString file =  proc.readAll();
-        QStringList content = file.split( "\n", QString::SkipEmptyParts );
-        if ( content.count() > 2 )
+        QStringList content = file.split( "\n" );
+        QStringListIterator it( content );
+        while ( it.hasNext() )
         {
-            for ( int i = 2; i < content.count(); i++ )
-            {
-                QString interface = content[i].simplified();
-                int index = interface.indexOf( ':' );
-                if ( index > -1 )
-                {
-                    interface = interface.left( index );
-                    if ( interface == "lo" )
-                        continue;
-                }
-
-                InterfaceSettings* settings = new InterfaceSettings();
-                settings->customCommands = false;
-                settings->hideWhenNotAvailable = false;
-                settings->hideWhenNotExisting = false;
-                settings->activateStatistics = false;
-                mSettingsDict.insert( interface, settings );
-                mDlg->listBoxInterfaces->addItem( interface );
-            }
-            if ( mDlg->listBoxInterfaces->count() > 0 )
-            {
-                mDlg->listBoxInterfaces->setCurrentRow( 0 );
-            }
-            else
-            {
-                mDlg->lineEditAlias->setText( QString::null );
-                mDlg->comboBoxIconSet->setCurrentIndex( 0 );
-                mDlg->checkBoxNotConnected->setChecked( false );
-                mDlg->checkBoxNotExisting->setChecked( false );
-                mDlg->checkBoxStatistics->setChecked( false );
-                mDlg->checkBoxCustom->setChecked( false );
-            }
+            QStringList routeParameter = it.next().split( QRegExp("\\s"), QString::SkipEmptyParts );
+            if ( routeParameter.count() < 2 )
+                continue;
+            if ( routeParameter[1] != "00000000" ) // no default route
+                continue;
+            if ( routeParameter[0] == "lo" )
+                continue;
+            QString interface = routeParameter[0]; // default route interface
+            InterfaceSettings* settings = new InterfaceSettings();
+            settings->customCommands = false;
+            settings->hideWhenNotAvailable = false;
+            settings->hideWhenNotExisting = false;
+            settings->activateStatistics = false;
+            settings->iconSet = "monitor";
+            settings->alias = interface;
+            mSettingsMap.insert( interface, settings );
+            mDlg->listBoxInterfaces->addItem( interface );
+            mDlg->lineEditAlias->setText( interface );
+            break;
+        }
+        if ( mDlg->listBoxInterfaces->count() > 0 )
+        {
+            mDlg->listBoxInterfaces->setCurrentRow( 0 );
+            mDlg->pushButtonDelete->setDisabled( false );
+            mDlg->groupBoxIfaceMisc->setDisabled( false );
+            mDlg->groupBoxIfaceMenu->setDisabled( false );
         }
         proc.close();
     }
@@ -521,13 +552,12 @@ void ConfigDialog::defaults()
 
     // Default plotter settings
     mDlg->spinBoxPixel->setValue( 1 );
-    mDlg->spinBoxCount->setValue( 5 );
     mDlg->spinBoxDistance->setValue( 30 );
     mDlg->spinBoxFontSize->setValue( 8 );
     mDlg->spinBoxMinValue->setValue( 0 );
     mDlg->spinBoxMaxValue->setValue( 1 );
     mDlg->checkBoxLabels->setChecked( true );
-    mDlg->checkBoxTopBar->setChecked( false );
+    mDlg->checkBoxBottomBar->setChecked( true );
     mDlg->checkBoxVLines->setChecked( true );
     mDlg->checkBoxHLines->setChecked( true );
     mDlg->checkBoxIncoming->setChecked( true );
@@ -555,8 +585,59 @@ void ConfigDialog::buttonNewSelected()
     {
         QListWidgetItem *item = new QListWidgetItem( ifname );
         mDlg->listBoxInterfaces->addItem( item );
-        mSettingsDict.insert( ifname, new InterfaceSettings() );
+        mSettingsMap.insert( ifname, new InterfaceSettings() );
         mDlg->listBoxInterfaces->setCurrentRow( mDlg->listBoxInterfaces->row( item ) );
+        mDlg->pushButtonDelete->setDisabled( false );
+        mDlg->groupBoxIfaceMisc->setDisabled( false );
+        mDlg->groupBoxIfaceMenu->setDisabled( false );
+        mDlg->lineEditAlias->setText( ifname );
+        changed( true );
+    }
+}
+
+void ConfigDialog::buttonAllSelected()
+{
+    QFile proc( "/proc/net/dev" );
+    if ( proc.open( QIODevice::ReadOnly ) )
+    {
+        QString file =  proc.readAll();
+        QStringList content = file.split( "\n", QString::SkipEmptyParts );
+        if ( content.count() > 2 )
+        {
+            for ( int i = 2; i < content.count(); i++ )
+            {
+                QString interface = content[i].simplified();
+                int index = interface.indexOf( ':' );
+                if ( index > -1 )
+                {
+                    interface = interface.left( index );
+                    if ( interface == "lo" || mSettingsMap.contains( interface ) )
+                        continue;
+                }
+
+                InterfaceSettings* settings = new InterfaceSettings();
+                settings->customCommands = false;
+                settings->hideWhenNotAvailable = false;
+                settings->hideWhenNotExisting = false;
+                settings->activateStatistics = false;
+                settings->iconSet = "monitor";
+                settings->alias = interface;
+                mSettingsMap.insert( interface, settings );
+                mDlg->listBoxInterfaces->addItem( interface );
+            }
+            if ( mDlg->listBoxInterfaces->count() > 0 )
+            {
+                mDlg->listBoxInterfaces->setCurrentRow( 0 );
+                mDlg->pushButtonDelete->setDisabled( false );
+                mDlg->groupBoxIfaceMisc->setDisabled( false );
+                mDlg->groupBoxIfaceMenu->setDisabled( false );
+                QString iface = mDlg->listBoxInterfaces->item( 0 )->text();
+                mDlg->lineEditAlias->blockSignals( true );
+                mDlg->lineEditAlias->setText( mSettingsMap[iface]->alias );
+                mDlg->lineEditAlias->blockSignals( false );
+            }
+        }
+        proc.close();
         changed( true );
     }
 }
@@ -573,12 +654,15 @@ void ConfigDialog::buttonDeleteSelected()
         mDeletedIfaces << selected->text();
 
     // TODO: find a better way than blocking signals
-    mSettingsDict.remove( selected->text() );
+    mSettingsMap.remove( selected->text() );
     mDlg->lineEditAlias->blockSignals( true );
     mDlg->lineEditAlias->setText( QString::null );
     mDlg->lineEditAlias->blockSignals( false );
     mDlg->comboBoxIconSet->blockSignals( true );
-    mDlg->comboBoxIconSet->setCurrentIndex( 0 );
+    if ( mIconSets.contains( "monitor" ) )
+        mDlg->comboBoxIconSet->setCurrentIndex( mIconSets.indexOf( "monitor" ) );
+    else
+        mDlg->comboBoxIconSet->setCurrentIndex( 0 );
     mDlg->comboBoxIconSet->blockSignals( false );
     mDlg->checkBoxNotConnected->blockSignals( true );
     mDlg->checkBoxNotConnected->setChecked( false );
@@ -594,6 +678,17 @@ void ConfigDialog::buttonDeleteSelected()
     mDlg->checkBoxCustom->blockSignals( false );
     QListWidgetItem *taken = mDlg->listBoxInterfaces->takeItem( mDlg->listBoxInterfaces->row( selected ) );
     delete taken;
+    if ( mDlg->listBoxInterfaces->count() < 1 )
+    {
+        mDlg->pushButtonDelete->setDisabled( true );
+        mDlg->groupBoxIfaceMisc->setDisabled( true );
+        mDlg->groupBoxIfaceMenu->setDisabled( true );
+        mDlg->pixmapDisconnected->clear();
+        mDlg->pixmapConnected->clear();
+        mDlg->pixmapIncoming->clear();
+        mDlg->pixmapOutgoing->clear();
+        mDlg->pixmapTraffic->clear();
+    }
     changed( true );
 }
 
@@ -604,7 +699,7 @@ void ConfigDialog::buttonAddCommandSelected()
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict.value( selected->text() );
+    InterfaceSettings* settings = mSettingsMap.value( selected->text() );
 
     InterfaceCommand cmd;
     cmd.runAsRoot = false;
@@ -633,7 +728,7 @@ void ConfigDialog::buttonRemoveCommandSelected()
     mDlg->listViewCommands->takeTopLevelItem( index );
     delete item;
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
 
     QList<InterfaceCommand> cmds;
     QTreeWidgetItemIterator i( mDlg->listViewCommands );
@@ -667,7 +762,7 @@ void ConfigDialog::buttonCommandUpSelected()
     mDlg->listViewCommands->insertTopLevelItem( index - 1, item );
     mDlg->listViewCommands->setCurrentItem( item );
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
 
     QList<InterfaceCommand> cmds;
     QTreeWidgetItemIterator i( mDlg->listViewCommands );
@@ -701,7 +796,7 @@ void ConfigDialog::buttonCommandDownSelected()
     mDlg->listViewCommands->insertTopLevelItem( index + 1, item );
     mDlg->listViewCommands->setCurrentItem( item );
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
 
     QList<InterfaceCommand> cmds;
     QTreeWidgetItemIterator i( mDlg->listViewCommands );
@@ -807,10 +902,13 @@ void ConfigDialog::interfaceSelected( int row )
     if ( row < 0 )
         return;
     QString interface = mDlg->listBoxInterfaces->item( row )->text();
-    InterfaceSettings* settings = mSettingsDict[interface];
+    InterfaceSettings* settings = mSettingsMap[interface];
     mLock = true;
     mDlg->lineEditAlias->setText( settings->alias );
-    mDlg->comboBoxIconSet->setCurrentIndex( settings->iconSet );
+    if ( mIconSets.contains( settings->iconSet ) )
+        mDlg->comboBoxIconSet->setCurrentIndex( mIconSets.indexOf( settings->iconSet ) );
+    else
+        mDlg->comboBoxIconSet->setCurrentIndex( 0 );
     mDlg->checkBoxCustom->setChecked( settings->customCommands );
     mDlg->checkBoxNotConnected->setChecked( settings->hideWhenNotAvailable );
     mDlg->checkBoxNotExisting->setChecked( settings->hideWhenNotExisting );
@@ -833,7 +931,11 @@ void ConfigDialog::interfaceSelected( int row )
     }
     mDlg->listViewCommands->addTopLevelItems( items );
 
-    iconSetChanged( settings->iconSet ); // to update iconset preview
+    // to update iconset preview
+    if ( mIconSets.contains( settings->iconSet ) )
+        iconSetChanged( mIconSets.indexOf( settings->iconSet ) );
+    else
+        iconSetChanged( 0 );
     mLock = false;
 }
 
@@ -844,7 +946,7 @@ void ConfigDialog::aliasChanged( const QString& text )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->alias = text;
     if (!mLock) changed( true );
 }
@@ -856,32 +958,16 @@ void ConfigDialog::iconSetChanged( int set )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
-    settings->iconSet = set;
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
+    settings->iconSet = mDlg->comboBoxIconSet->itemText( set );
 
     // Update the preview of the iconset.
-    QString suffix;
-    switch ( set )
-    {
-    case NETWORK:
-        suffix = SUFFIX_LAN;
-        break;
-    case WIRELESS:
-        suffix = SUFFIX_WLAN;
-        break;
-    case MODEM:
-        suffix = SUFFIX_PPP;
-        break;
-    default:
-        suffix = ""; // use standard icons
-    }
-
     KIconLoader::global()->addAppDir( "knemo" );
-    mDlg->pixmapDisconnected->setPixmap( UserIcon( ICON_DISCONNECTED + suffix ) );
-    mDlg->pixmapConnected->setPixmap( UserIcon( ICON_CONNECTED + suffix ) );
-    mDlg->pixmapIncoming->setPixmap( UserIcon( ICON_INCOMING + suffix ) );
-    mDlg->pixmapOutgoing->setPixmap( UserIcon( ICON_OUTGOING + suffix ) );
-    mDlg->pixmapTraffic->setPixmap( UserIcon( ICON_TRAFFIC + suffix ) );
+    mDlg->pixmapDisconnected->setPixmap( UserIcon( settings->iconSet + ICON_DISCONNECTED ) );
+    mDlg->pixmapConnected->setPixmap( UserIcon( settings->iconSet + ICON_CONNECTED ) );
+    mDlg->pixmapIncoming->setPixmap( UserIcon( settings->iconSet + ICON_INCOMING ) );
+    mDlg->pixmapOutgoing->setPixmap( UserIcon( settings->iconSet + ICON_OUTGOING ) );
+    mDlg->pixmapTraffic->setPixmap( UserIcon( settings->iconSet + ICON_TRAFFIC ) );
     if (!mLock) changed( true );
 }
 
@@ -898,7 +984,7 @@ void ConfigDialog::checkBoxNotConnectedToggled( bool on )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->hideWhenNotAvailable = on;
     if (!mLock) changed( true );
 }
@@ -910,7 +996,7 @@ void ConfigDialog::checkBoxNotExistingToggled( bool on )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->hideWhenNotExisting = on;
     if (!mLock) changed( true );
 }
@@ -923,7 +1009,7 @@ void ConfigDialog::checkBoxStatisticsToggled( bool on )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->activateStatistics = on;
     if (!mLock) changed( true );
 
@@ -939,9 +1025,6 @@ void ConfigDialog::checkBoxStartKNemoToggled( bool on )
         KConfigGroup generalGroup( config, "General" );
         if ( generalGroup.readEntry( "FirstStart", true ) )
         {
-            generalGroup.writeEntry( "FirstStart", false );
-            config->sync();
-
             // Populate the dialog with some default values if the user starts
             // KNemo for the very first time.
             defaults();
@@ -958,7 +1041,7 @@ void ConfigDialog::spinBoxTrafficValueChanged( int value )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->trafficThreshold = value;
     if (!mLock) changed( true );
 }
@@ -970,7 +1053,7 @@ void ConfigDialog::checkBoxCustomToggled( bool on )
 
     QListWidgetItem* selected = mDlg->listBoxInterfaces->currentItem();
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     settings->customCommands = on;
     if ( on )
     {
@@ -1046,7 +1129,7 @@ void ConfigDialog::setupToolTipArray()
 void ConfigDialog::updateStatisticsEntries( void )
 {
     bool statisticsActive = false;
-    foreach ( InterfaceSettings *it, mSettingsDict )
+    foreach ( InterfaceSettings *it, mSettingsMap )
     {
         if ( it->activateStatistics )
         {
@@ -1056,7 +1139,6 @@ void ConfigDialog::updateStatisticsEntries( void )
     }
 
     mDlg->groupBoxStatistics->setEnabled( statisticsActive );
-    mDlg->groupBoxUpdateInterval->setEnabled( statisticsActive );
 }
 
 void ConfigDialog::checkBoxToggled( bool )
@@ -1091,7 +1173,7 @@ void ConfigDialog::listViewCommandsChanged( QTreeWidgetItem* item, int column )
 
     int row = mDlg->listViewCommands->indexOfTopLevelItem( item );
 
-    InterfaceSettings* settings = mSettingsDict[selected->text()];
+    InterfaceSettings* settings = mSettingsMap[selected->text()];
     InterfaceCommand& cmd = settings->commands[row];
     switch ( column )
     {
