@@ -112,6 +112,36 @@ void NetToolsBackend::update()
 #endif
 }
 
+QString NetToolsBackend::getDefaultRouteIface()
+{
+    QString iface;
+#ifdef PATH_ROUTE
+    KProcess droute;
+    droute.setOutputChannelMode( KProcess::MergedChannels );
+    droute.setEnv( "LANG", "C" );
+    droute.setEnv( "LC_ALL", "C" );
+    droute << PATH_ROUTE << "-n";
+    droute.execute();
+    QString routeStdout = droute.readAllStandardOutput();
+
+    QStringList routeList = routeStdout.split( "\n" );
+    QStringListIterator it( routeList );
+    while ( it.hasNext() )
+    {
+        QStringList routeParameter = it.next().split( " ", QString::SkipEmptyParts );
+        if ( routeParameter.count() < 8 ) // no routing entry
+            continue;
+        if ( routeParameter[0] != "0.0.0.0" ) // no default route
+            continue;
+        if ( routeParameter[7] == "lo" )
+            continue;
+        iface = routeParameter[7]; // default route interface
+        break;
+    }
+#endif
+    return iface;
+}
+
 void NetToolsBackend::routeProcessExited( int, QProcess::ExitStatus )
 {
     mRouteProcess->deleteLater(); // we're in a slot connected to mRouteProcess
@@ -349,12 +379,12 @@ void NetToolsBackend::parseIwconfigOutput()
         {
             // Update the wireless data of the interface.
             interface->getData().wirelessDevice = true;
-            updateWirelessData( configs[key], interface->getWirelessData() );
+            updateWirelessData( key, configs[key], interface->getWirelessData() );
         }
     }
 }
 
-void NetToolsBackend::updateWirelessData( QString& config, WirelessData& data )
+void NetToolsBackend::updateWirelessData( QString& ifaceName, QString& config, WirelessData& data )
 {
     QRegExp regExp( "ESSID:([^\"][\\S]*)" );
     if ( regExp.indexIn( config ) > -1 )
@@ -395,31 +425,71 @@ void NetToolsBackend::updateWirelessData( QString& config, WirelessData& data )
     regExp.setPattern( "(.{2}:.{2}:.{2}:.{2}:.{2}:.{2})" );
     if ( regExp.indexIn( config ) > -1 )
         data.accessPoint = regExp.cap( 1 );
+    else
+        data.accessPoint.clear();
 
     regExp.setPattern( "Nickname:\"(\\w*)\"" );
     if ( regExp.indexIn( config ) > -1 )
         data.nickName = regExp.cap( 1 );
 
-    regExp.setPattern( "Link Quality[=:]([\\d]*)" );
-    if ( regExp.indexIn( config ) > -1 )
-        data.linkQuality = regExp.cap( 1 );
-
-    regExp.setPattern( "Encryption key:" );
+    regExp.setPattern( "Link Quality[=:]([\\d]*)(\\/([\\d]+))?" );
     if ( regExp.indexIn( config ) > -1 )
     {
-        regExp.setPattern( "Encryption key:off" );
-        if ( regExp.indexIn( config ) > -1 )
+        if ( regExp.numCaptures() == 3 && !regExp.cap( 3 ).isEmpty() )
         {
-            data.encryption = false;
+            int maxQual = regExp.cap( 3 ).toInt();
+            if ( maxQual > 0 )
+                data.linkQuality = QString( "%1%" ).arg( 100 * regExp.cap( 1 ).toInt() / maxQual );
+            else
+                data.linkQuality = "0";
         }
         else
-        {
-            data.encryption = true;
-        }
+            data.linkQuality = regExp.cap( 1 );
     }
-    else
+
+#ifdef PATH_IWLIST
+    if ( data.accessPoint != data.prevAccessPoint )
     {
+        /* Reset encryption status for new access point */
         data.encryption = false;
+        data.prevAccessPoint = data.accessPoint;
+    }
+    /* We only use left-over wireless scans to prevent doing a new scan every
+     * polling period.  This requires that we run iwlist once per wireless
+     * device.  If our current access point disappears from the results then
+     * parseWirelessEncryption will use the last encryption status until the
+     * results are updated again. */
+    KProcess iwlistProcess;
+    iwlistProcess.setOutputChannelMode( KProcess::MergedChannels );
+    iwlistProcess.setEnv( "LANG", "C" );
+    iwlistProcess.setEnv( "LC_ALL", "C" );
+    iwlistProcess << PATH_IWLIST << ifaceName << "scan" << "last";
+
+    iwlistProcess.execute();
+    QString iwlistOutput = iwlistProcess.readAllStandardOutput();
+    parseWirelessEncryption( iwlistOutput, data );
+#endif
+}
+
+void NetToolsBackend::parseWirelessEncryption( QString& config, WirelessData& data )
+{
+    QStringList apList = config.split( "Cell [0-9]{2} - ", QString::SkipEmptyParts );
+    foreach( QString ap, apList )
+    {
+        QRegExp regExp( "Address: (.{2}:.{2}:.{2}:.{2}:.{2}:.{2})" );
+        if ( regExp.indexIn( ap ) > -1 && regExp.cap( 1 ) == data.accessPoint )
+        {
+            regExp.setPattern( "Encryption key:" );
+            if ( regExp.indexIn( ap ) > -1 )
+            {
+                regExp.setPattern( "Encryption key:off" );
+                if ( regExp.indexIn( ap ) > -1 )
+                    data.encryption = false;
+                else
+                    data.encryption = true;
+                return;
+            }
+        }
     }
 }
 
