@@ -45,6 +45,7 @@ public:
     static BackendBase* createInstance( QHash<QString, Interface *>& interfaces );
 
     void update();
+    QString getDefaultRouteIface();
 
 private slots:
     void routeProcessExited( int exitCode, QProcess::ExitStatus status );
@@ -59,7 +60,8 @@ private:
     void parseIfconfigOutput();
     void updateInterfaceData( QString& config, InterfaceData& data, int type );
     void parseIwconfigOutput();
-    void updateWirelessData( QString& config, WirelessData& data );
+    void updateWirelessData( QString& ifaceName, QString& config, WirelessData& data );
+    void parseWirelessEncryption( QString& config, WirelessData& data );
 
     QString mRouteStdout;
     QString mIfconfigStdout;
