@@ -24,6 +24,8 @@
 #include <QColor>
 #include <QString>
 
+#include <KUrl>
+
 /**
  * This file contains data structures and enums used in the knemo daemon.
  *
@@ -36,7 +38,7 @@ struct GeneralData
     int toolTipContent;
     int pollInterval;
     int saveInterval;
-    QString statisticsDir;
+    KUrl statisticsDir;
 };
 
 struct InterfaceData
@@ -89,6 +91,7 @@ struct WirelessData
     QString bitRate;
     QString linkQuality;
     QString accessPoint;
+    QString prevAccessPoint;
     QString nickName;
     bool encryption;
 };
@@ -96,13 +99,12 @@ struct WirelessData
 struct PlotterSettings
 {
     int pixel;
-    int count;
     int distance;
     int fontSize;
     int minimumValue;
     int maximumValue;
     bool labels;
-    bool topBar;
+    bool bottomBar;
     bool showIncoming;
     bool showOutgoing;
     bool verticalLines;
