@@ -44,6 +44,11 @@ Interface::Interface( QString ifname,
       mGeneralData( generalData ),
       mPlotterSettings( plotterSettings )
 {
+    mPlotterDialog = new InterfacePlotterDialog( mPlotterSettings, mName );
+    mPlotterTimer = new QTimer();
+
+    connect( mPlotterTimer, SIGNAL( timeout() ),
+             this, SLOT( updatePlotter() ) );
     connect( &mMonitor, SIGNAL( statusChanged( int ) ),
              &mIcon, SLOT( updateStatus( int ) ) );
     connect( &mMonitor, SIGNAL( available( int ) ),
@@ -90,7 +95,6 @@ void Interface::configChanged()
     mIcon.updateTrayStatus( UNKNOWN_STATE );
     // handle changed iconset by user
     mIcon.updateStatus( mState );
-    mIcon.updateToolTipTime( QString () );
     mIcon.updateMenu();
 
     if ( mPlotterDialog != 0L )
@@ -127,8 +131,10 @@ void Interface::activateMonitor()
 
 void Interface::setStartTime( int )
 {
-    mStartTime.setDate( QDate::currentDate() );
-    mStartTime.setTime( QTime::currentTime() );
+    mUptime = 0;
+    mUptimeString = "00:00:00";
+    if ( !mPlotterTimer->isActive() )
+        mPlotterTimer->start( 1000 );
 }
 
 void Interface::showStatusDialog()
@@ -161,32 +167,18 @@ void Interface::showStatusDialog()
 
 void Interface::showSignalPlotter( bool wasMiddleButton )
 {
-    // No plotter, create it.
-    if ( mPlotterDialog == 0L )
+    if ( wasMiddleButton )
     {
-        mPlotterDialog = new InterfacePlotterDialog( mPlotterSettings, mName );
-        //mPlotter->setTitle( mName );
-        mPlotterDialog->configChanged();
-        activateOrHide( mPlotterDialog, true );
-
-        mPlotterTimer = new QTimer();
-        connect( mPlotterTimer, SIGNAL( timeout() ),
-                 this, SLOT( updatePlotter() ) );
-        mPlotterTimer->start( 1000 );
+        // Toggle the signal plotter.
+        activateOrHide( mPlotterDialog );
     }
     else
     {
-        if ( wasMiddleButton )
-        {
-            // Toggle the signal plotter.
-            activateOrHide( mPlotterDialog );
-        }
-        else
-        {
-            // Called from the context menu, show the dialog.
-            activateOrHide( mPlotterDialog, true );
-        }
+        // Called from the context menu, show the dialog.
+        activateOrHide( mPlotterDialog, true );
     }
+    if ( mPlotterDialog->isVisible() && !mPlotterTimer->isActive() )
+        mPlotterTimer->start( 1000 );
 }
 
 void Interface::showStatisticsDialog()
@@ -214,11 +206,17 @@ void Interface::showStatisticsDialog()
         connect( mStatisticsDialog, SIGNAL( clearYearlyStatisticsClicked() ),
                  mStatistics, SLOT( clearYearStatistics() ) );
 
+        /* We need to show the dialog before we update the stats.  That way
+         * the viewport doesn't get messed up when we scroll to the most recent
+         * item. */
+        mStatisticsDialog->show();
+
         mStatisticsDialog->updateDays();
         mStatisticsDialog->updateMonths();
         mStatisticsDialog->updateYears();
     }
-    mStatisticsDialog->show();
+    else
+        mStatisticsDialog->show();
 }
 
 void Interface::resetData( int state )
@@ -242,12 +240,12 @@ void Interface::updateDetails()
 {
     InterfaceData &data = getData();
 
+    mUptime += mGeneralData.pollInterval;
     QString uptime;
     if ( data.available )
     {
-        int upsecs = getStartTime().secsTo( QDateTime::currentDateTime() );
-        int updays = upsecs / 86400; // don't use QDateTime::daysTo() because
-                                     // we only want complete days
+        time_t upsecs = mUptime;
+        time_t updays = upsecs / 86400;
 
         if ( updays == 1 )
             uptime = "1 day, ";
@@ -262,8 +260,9 @@ void Interface::updateDetails()
         time.sprintf( "%02d:%02d:%02d", hrs, mins, secs );
         uptime += time;
     }
+    mUptimeString = uptime;
 
-    mIcon.updateToolTipTime( uptime );
+    mIcon.updateToolTip();
     if ( mStatusDialog )
         mStatusDialog->updateDialog();
 }
