@@ -78,9 +78,14 @@ public:
         return mState;
     }
 
-    const QDateTime& getStartTime() const
+    time_t getUptime()
     {
-        return mStartTime;
+        return mUptime;
+    }
+
+    QString getUptimeString()
+    {
+        return mUptimeString;
     }
 
     const QString& getName() const
@@ -214,7 +219,8 @@ private:
     int mState;
     QString mName;
     QTimer* mPlotterTimer;
-    QDateTime mStartTime;
+    time_t mUptime;
+    QString mUptimeString;
     InterfaceIcon mIcon;
     InterfaceData mData;
     InterfaceMonitor mMonitor;
