@@ -35,16 +35,14 @@
 
 InterfaceStatisticsDialog::InterfaceStatisticsDialog( Interface* interface, QWidget* parent )
     : KDialog( parent ),
+      mSetPos( true ),
       mConfig( KGlobal::config() ),
       mInterface( interface )
 {
     setCaption( interface->getName() + " " + i18n( "Statistics" ) );
     setButtons( Close );
 
-    QWidget *main = new QWidget( this );
-    setMainWidget( main );
-    ui.setupUi( main );
-    setMainWidget( main );
+    ui.setupUi( mainWidget() );
 
     dailyModel = new QStandardItemModel( this );
     monthlyModel = new QStandardItemModel( this );
@@ -66,34 +64,48 @@ InterfaceStatisticsDialog::InterfaceStatisticsDialog( Interface* interface, QWid
 
     // Restore window size and position.
     KConfig *config = mConfig.data();
-    KConfigGroup plotterGroup = config->group( "Interface_" + mInterface->getName() );
-    mGeometry = plotterGroup.readEntry( "StatisticsGeometry", mGeometry );
-
-    // A bit of a hack. The first time around, the size/position doesn't seem
-    // to stick between show/hide events unless we explicitly set it on startup
-    // Is there a better way?
-    if ( mGeometry.isEmpty() )
+    KConfigGroup interfaceGroup( config, "Interface_" + mInterface->getName() );
+    if ( interfaceGroup.hasKey( "StatisticsPos" ) )
     {
-        resize( sizeHint() );
-        move( pos() );
+        QPoint p = interfaceGroup.readEntry( "StatisticsPos", QPoint() );
+        // See comment in event()
+        mSetPos = false;
+        move( p );
+    }
+    if ( interfaceGroup.hasKey( "StatisticsSize" ) )
+    {
+        QSize s = interfaceGroup.readEntry( "StatisticsSize", QSize() );
+        resize( s );
     }
     else
-        restoreGeometry( mGeometry );
+    {
+        // Improve the chance that we have a decent sized dialog
+        // the first time it's shown
+        resize( 600, 450 );
+    }
 }
 
 InterfaceStatisticsDialog::~InterfaceStatisticsDialog()
 {
     KConfig *config = mConfig.data();
     KConfigGroup interfaceGroup( config, "Interface_" + mInterface->getName() );
-    interfaceGroup.writeEntry( "StatisticsGeometry", saveGeometry() );
+    interfaceGroup.writeEntry( "StatisticsPos", pos() );
+    interfaceGroup.writeEntry( "StatisticsSize", size() );
     config->sync();
 }
 
 bool InterfaceStatisticsDialog::event( QEvent *e )
 {
-    if ( e->type() == QEvent::Move || e->type() == QEvent::Resize )
+    /* If we do not explicitly call size() and move() at least once then
+     * hiding and showing the dialog will cause it to forget its previous
+     * size and position. */
+    if ( e->type() == QEvent::Move )
     {
-        mGeometry = saveGeometry();
+        if ( mSetPos && !pos().isNull() )
+        {
+            mSetPos = false;
+            move( pos() );
+        }
     }
 
     return KDialog::event( e );
@@ -120,8 +132,12 @@ void InterfaceStatisticsDialog::updateDays()
 
     dailyModel->setVerticalHeaderLabels( vheaders );
 
-    //ui.tableDaily->setCurrentCell( row - 1, 2 );
-    //ui.tableDaily->ensureCellVisible( row - 1, 2 );
+    if ( dailyModel->rowCount() > 0 )
+    {
+        QModelIndex scrollIndex = dailyModel->item( dailyModel->rowCount() - 1, 2 )->index();
+        ui.tableDaily->setCurrentIndex( scrollIndex );
+        ui.tableDaily->scrollTo( scrollIndex );
+    }
 }
 
 void InterfaceStatisticsDialog::updateMonths()
@@ -145,8 +161,12 @@ void InterfaceStatisticsDialog::updateMonths()
 
     monthlyModel->setVerticalHeaderLabels( vheaders );
 
-    //ui.tableMonthly->setCurrentCell( row - 1, 2 );
-    //ui.tableMonthly->ensureCellVisible( row - 1, 2 );
+    if ( monthlyModel->rowCount() > 0 )
+    {
+        QModelIndex scrollIndex = monthlyModel->item( monthlyModel->rowCount() - 1, 2 )->index();
+        ui.tableMonthly->setCurrentIndex( scrollIndex );
+        ui.tableMonthly->scrollTo( scrollIndex );
+    }
 }
 
 void InterfaceStatisticsDialog::updateYears()
@@ -169,8 +189,12 @@ void InterfaceStatisticsDialog::updateYears()
 
     yearlyModel->setVerticalHeaderLabels( vheaders );
 
-    //tableYearly->setCurrentCell( row - 1, 2 );
-    //tableYearly->ensureCellVisible( row - 1, 2 );
+    if ( yearlyModel->rowCount() > 0 )
+    {
+        QModelIndex scrollIndex = yearlyModel->item( yearlyModel->rowCount() - 1, 2 )->index();
+        ui.tableYearly->setCurrentIndex( scrollIndex );
+        ui.tableYearly->scrollTo( scrollIndex );
+    }
 }
 
 void InterfaceStatisticsDialog::updateCurrentEntry()
