@@ -27,16 +27,14 @@
 
 InterfaceStatusDialog::InterfaceStatusDialog( Interface* interface, QWidget* parent )
     : KDialog( parent ),
+      mSetPos( true ),
       mConfig( KGlobal::config() ),
       mInterface( interface )
 {
     setCaption( interface->getName() + " " + i18n( "Interface Status" ) );
     setButtons( Close );
 
-    QWidget *main = new QWidget( this );
-    setMainWidget( main );
-    ui.setupUi( main );
-    setMainWidget( main );
+    ui.setupUi( mainWidget() );
 
     updateDialog();
     if ( interface->getData().available )
@@ -62,18 +60,20 @@ InterfaceStatusDialog::InterfaceStatusDialog( Interface* interface, QWidget* par
     // Restore window size and position.
     KConfig *config = mConfig.data();
     KConfigGroup interfaceGroup( config, "Interface_" + mInterface->getName() );
-    mGeometry = interfaceGroup.readEntry( "StatusGeometry", mGeometry );
-
-    // A bit of a hack. The first time around, the size/position doesn't seem
-    // to stick between show/hide events unless we explicitly set it on startup
-    // Is there a better way?
-    if ( mGeometry.isEmpty() )
+    if ( interfaceGroup.hasKey( "StatusPos" ) )
     {
-        resize( sizeHint() );
-        move( pos() );
+        QPoint p = interfaceGroup.readEntry( "StatusPos", QPoint() );
+        // See comment in event()
+        mSetPos = false;
+        move( p );
+    }
+    if ( interfaceGroup.hasKey( "StatusSize" ) )
+    {
+        QSize s = interfaceGroup.readEntry( "StatusSize", QSize() );
+        resize( s );
     }
     else
-        restoreGeometry( mGeometry );
+        resize( sizeHint() );
 
     statisticsChanged();
 }
@@ -82,15 +82,23 @@ InterfaceStatusDialog::~InterfaceStatusDialog()
 {
     KConfig *config = mConfig.data();
     KConfigGroup interfaceGroup( config, "Interface_" + mInterface->getName() );
-    interfaceGroup.writeEntry( "StatusGeometry", mGeometry );
+    interfaceGroup.writeEntry( "StatusPos", pos() );
+    interfaceGroup.writeEntry( "StatusSize", size() );
     config->sync();
 }
 
 bool InterfaceStatusDialog::event( QEvent *e )
 {
-    if ( e->type() == QEvent::Move || e->type() == QEvent::Resize )
+    /* If we do not explicitly call size() and move() at least once then
+     * hiding and showing the dialog will cause it to forget its previous
+     * size and position. */
+    if ( e->type() == QEvent::Move )
     {
-        mGeometry = saveGeometry();
+        if ( mSetPos && !pos().isNull() )
+        {
+            mSetPos = false;
+            move( pos() );
+        }
     }
 
     return KDialog::event( e );
@@ -112,9 +120,8 @@ void InterfaceStatusDialog::updateDialog()
     if ( data.available )
     {
         ui.textLabelStatus->setText( i18n( "Connection established." ) );
-        int upsecs = mInterface->getStartTime().secsTo( QDateTime::currentDateTime() );
-        int updays = upsecs / 86400; // don't use QDateTime::daysTo() because
-                                     // we only want complete days
+        time_t upsecs = mInterface->getUptime();
+        time_t updays = upsecs / 86400;
 
         QString uptime;
         if ( updays == 1 )
@@ -197,7 +204,7 @@ void InterfaceStatusDialog::updateDialog()
             ui.textLabelMode->setText( wdata.mode );
             ui.textLabelFreqChannel->setText( wdata.frequency + " [" + wdata.channel + "]" );
             ui.textLabelBitRate->setText( wdata.bitRate );
-            ui.textLabelLinkQuality->setText( wdata.linkQuality + "%" );
+            ui.textLabelLinkQuality->setText( wdata.linkQuality );
             if ( wdata.encryption == true )
             {
                 ui.textLabelEncryption->setText( i18n( "active" ) );
