@@ -37,11 +37,13 @@ QString KNemoDaemon::sSelectedInterface = QString::null;
 KNemoDaemon::KNemoDaemon()
     : QObject(),
       mConfig( KGlobal::config() ),
+      mHaveInterfaces( false ),
       mColorVLines( 0x04FB1D ),
       mColorHLines( 0x04FB1D ),
       mColorIncoming( 0x1889FF ),
       mColorOutgoing( 0xFF7F08 ),
-      mColorBackground( 0x313031 )
+      mColorBackground( 0x313031 ),
+      mBackend( 0 )
 {
     readConfig();
     QDBusConnection::sessionBus().registerObject("/knemo", this, QDBusConnection::ExportScriptableSlots);
@@ -59,11 +61,9 @@ KNemoDaemon::KNemoDaemon()
 
     if ( !foundBackend )
     {
-        i = 0; // use the first backend (Nettools)
+        i = 0; // use the first backend (Sys)
     }
-    mBackend = ( *DaemonRegistry[i].function )( mInterfaceDict );
-
-    //mInterfaceDict.setAutoDelete( true );
+    mBackend = ( *DaemonRegistry[i].function )( mInterfaceHash );
 
     mPollTimer = new QTimer();
     connect( mPollTimer, SIGNAL( timeout() ), this, SLOT( updateInterfaces() ) );
@@ -74,11 +74,12 @@ KNemoDaemon::~KNemoDaemon()
 {
     mPollTimer->stop();
     delete mPollTimer;
-    delete mBackend;
+    if ( mBackend )
+        delete mBackend;
 
-    foreach ( QString key, mInterfaceDict.keys() )
+    foreach ( QString key, mInterfaceHash.keys() )
     {
-        Interface *interface = mInterfaceDict.take( key );
+        Interface *interface = mInterfaceHash.take( key );
         delete interface;
     }
 }
@@ -87,97 +88,21 @@ void KNemoDaemon::readConfig()
 {
     KConfig *config = mConfig.data();
 
-    KConfigGroup generalGroup( config, "General" );
-    mBackendName = generalGroup.readEntry( "Backend", "Nettools" );
-    mGeneralData.pollInterval = generalGroup.readEntry( "PollInterval", 1 );
-    mGeneralData.saveInterval = generalGroup.readEntry( "SaveInterval", 60 );
-    mGeneralData.statisticsDir = generalGroup.readEntry( "StatisticsDir", KGlobal::dirs()->saveLocation( "data", "knemo/" ) );
-    mGeneralData.toolTipContent = generalGroup.readEntry( "ToolTipContent", 2 );
-    QStringList list = generalGroup.readEntry( "Interfaces", QStringList() );
-    int numEntries = list.count();
-
-    if ( numEntries == 0 )
-        return;
-
-    QString interface;
-    foreach ( QString interface, list )
-    {
-        Interface* iface = new Interface( interface, mGeneralData, mPlotterSettings );
-        QString group( "Interface_" );
-        group += interface;
-        if ( config->hasGroup( group ) )
-        {
-            KConfigGroup ifaceGroup( config, group );
-            InterfaceSettings& settings = iface->getSettings();
-            settings.alias = ifaceGroup.readEntry( "Alias" );
-            settings.iconSet = ifaceGroup.readEntry( "IconSet", 0 );
-            settings.customCommands = ifaceGroup.readEntry( "CustomCommands", false );
-            settings.hideWhenNotAvailable = ifaceGroup.readEntry( "HideWhenNotAvailable", false );
-            settings.hideWhenNotExisting = ifaceGroup.readEntry( "HideWhenNotExisting", false );
-            settings.activateStatistics = ifaceGroup.readEntry( "ActivateStatistics", false );
-            settings.trafficThreshold = ifaceGroup.readEntry( "TrafficThreshold", 0 );
-            if ( settings.customCommands )
-            {
-                int numCommands = ifaceGroup.readEntry( "NumCommands", 0 );
-                for ( int i = 0; i < numCommands; i++ )
-                {
-                    QString entry;
-                    InterfaceCommand cmd;
-                    entry = QString( "RunAsRoot%1" ).arg( i + 1 );
-                    cmd.runAsRoot = ifaceGroup.readEntry( entry, false );
-                    entry = QString( "Command%1" ).arg( i + 1 );
-                    cmd.command = ifaceGroup.readEntry( entry );
-                    entry = QString( "MenuText%1" ).arg( i + 1 );
-                    cmd.menuText = ifaceGroup.readEntry( entry );
-                    settings.commands.append( cmd );
-                }
-            }
-            iface->configChanged(); // important to activate the statistics
-        }
-        mInterfaceDict.insert( interface, iface );
-    }
-
-    // load the settings for the plotter
-    KConfigGroup plotterGroup( config, "PlotterSettings" );
-    mPlotterSettings.pixel = plotterGroup.readEntry( "Pixel", 1 );
-    mPlotterSettings.count = plotterGroup.readEntry( "Count", 5 );
-    mPlotterSettings.distance = plotterGroup.readEntry( "Distance", 30 );
-    mPlotterSettings.fontSize = plotterGroup.readEntry( "FontSize", 8 );
-    mPlotterSettings.minimumValue = plotterGroup.readEntry( "MinimumValue", 0 );
-    mPlotterSettings.maximumValue = plotterGroup.readEntry( "MaximumValue", 1 );
-    mPlotterSettings.labels = plotterGroup.readEntry( "Labels", true );
-    mPlotterSettings.topBar = plotterGroup.readEntry( "TopBar", false );
-    mPlotterSettings.showIncoming = plotterGroup.readEntry( "ShowIncoming", true );
-    mPlotterSettings.showOutgoing = plotterGroup.readEntry( "ShowOutgoing", true );
-    mPlotterSettings.verticalLines = plotterGroup.readEntry( "VerticalLines", true );
-    mPlotterSettings.horizontalLines = plotterGroup.readEntry( "HorizontalLines", true );
-    mPlotterSettings.automaticDetection = plotterGroup.readEntry( "AutomaticDetection", true );
-    mPlotterSettings.verticalLinesScroll = plotterGroup.readEntry( "VerticalLinesScroll", true );
-    mPlotterSettings.colorVLines = plotterGroup.readEntry( "ColorVLines", mColorVLines.rgb() );
-    mPlotterSettings.colorHLines = plotterGroup.readEntry( "ColorHLines", mColorHLines.rgb() );
-    mPlotterSettings.colorIncoming = plotterGroup.readEntry( "ColorIncoming", mColorIncoming.rgb() );
-    mPlotterSettings.colorOutgoing = plotterGroup.readEntry( "ColorOutgoing", mColorOutgoing.rgb() );
-    mPlotterSettings.colorBackground = plotterGroup.readEntry( "ColorBackground", mColorBackground.rgb() );
-}
-
-void KNemoDaemon::reparseConfiguration()
-{
-    // Read the settings from the config file.
-    QHash< QString, InterfaceSettings *> settingsDict;
-    KConfig *config = mConfig.data();
+    // For when reparseConfiguration() is called
     config->reparseConfiguration();
 
+    // General
     KConfigGroup generalGroup( config, "General" );
-    mGeneralData.pollInterval = generalGroup.readEntry( "PollInterval", 1 );
-    mGeneralData.saveInterval = generalGroup.readEntry( "SaveInterval", 60 );
+    mGeneralData.pollInterval = clamp<int>(generalGroup.readEntry( "PollInterval", 1 ), 1, 60 );
+    mGeneralData.saveInterval = clamp<int>(generalGroup.readEntry( "SaveInterval", 60 ), 1, 300 );
     mGeneralData.statisticsDir = generalGroup.readEntry( "StatisticsDir", KGlobal::dirs()->saveLocation( "data", "knemo/" ) );
     mGeneralData.toolTipContent = generalGroup.readEntry( "ToolTipContent", 2 );
-
-    // restart the timer with the new interval
-    mPollTimer->setInterval( mGeneralData.pollInterval * 1000 );
-
-    // select the backend from the config file
-    QString backend = generalGroup.readEntry( "Backend", "Nettools" );
+    // If we already have an Interfaces key--even if its empty--then we
+    // shouldn't try to set up a default interface
+    if ( generalGroup.hasKey( "Interfaces" ) )
+        mHaveInterfaces = true;
+    QStringList interfaceList = generalGroup.readEntry( "Interfaces", QStringList() );
+    QString backend = generalGroup.readEntry( "Backend", "Sys" );
 
     if ( mBackendName != backend )
     {
@@ -195,19 +120,36 @@ void KNemoDaemon::reparseConfiguration()
 
         if ( foundBackend )
         {
-            delete mBackend;
-            mBackend = ( *DaemonRegistry[i].function )( mInterfaceDict );
+            if ( mBackend )
+                delete mBackend;
+            mBackend = ( *DaemonRegistry[i].function )( mInterfaceHash );
         }
     }
 
-    QStringList list;
-    list = generalGroup.readEntry( "Interfaces", list );
-    int numEntries = list.count();
+    // Plotter
+    KConfigGroup plotterGroup( config, "PlotterSettings" );
+    mPlotterSettings.pixel = clamp<int>(plotterGroup.readEntry( "Pixel", 1 ), 1, 50 );
+    mPlotterSettings.distance = clamp<int>(plotterGroup.readEntry( "Distance", 30 ), 10, 120 );
+    mPlotterSettings.fontSize = clamp<int>(plotterGroup.readEntry( "FontSize", 8 ), 5, 24 );
+    mPlotterSettings.minimumValue = clamp<int>(plotterGroup.readEntry( "MinimumValue", 0 ), 0, 49999 );
+    mPlotterSettings.maximumValue = clamp<int>(plotterGroup.readEntry( "MaximumValue", 1 ), 1, 50000 );
+    mPlotterSettings.labels = plotterGroup.readEntry( "Labels", true );
+    mPlotterSettings.bottomBar = plotterGroup.readEntry( "BottomBar", false );
+    mPlotterSettings.showIncoming = plotterGroup.readEntry( "ShowIncoming", true );
+    mPlotterSettings.showOutgoing = plotterGroup.readEntry( "ShowOutgoing", true );
+    mPlotterSettings.verticalLines = plotterGroup.readEntry( "VerticalLines", true );
+    mPlotterSettings.horizontalLines = plotterGroup.readEntry( "HorizontalLines", true );
+    mPlotterSettings.automaticDetection = plotterGroup.readEntry( "AutomaticDetection", true );
+    mPlotterSettings.verticalLinesScroll = plotterGroup.readEntry( "VerticalLinesScroll", true );
+    mPlotterSettings.colorVLines = plotterGroup.readEntry( "ColorVLines", mColorVLines );
+    mPlotterSettings.colorHLines = plotterGroup.readEntry( "ColorHLines", mColorHLines );
+    mPlotterSettings.colorIncoming = plotterGroup.readEntry( "ColorIncoming", mColorIncoming );
+    mPlotterSettings.colorOutgoing = plotterGroup.readEntry( "ColorOutgoing", mColorOutgoing );
+    mPlotterSettings.colorBackground = plotterGroup.readEntry( "ColorBackground", mColorBackground );
 
-    if ( numEntries == 0 )
-        return;
-
-    foreach ( QString interface, list )
+    // Interfaces
+    QHash< QString, InterfaceSettings *> settingsHash;
+    foreach ( QString interface, interfaceList )
     {
         QString group( "Interface_" );
         group += interface;
@@ -216,12 +158,12 @@ void KNemoDaemon::reparseConfiguration()
         {
             KConfigGroup interfaceGroup( config, group );
             settings->alias = interfaceGroup.readEntry( "Alias" );
-            settings->iconSet = interfaceGroup.readEntry( "IconSet", 0 );
+            settings->iconSet = interfaceGroup.readEntry( "IconSet", "monitor" );
             settings->customCommands = interfaceGroup.readEntry( "CustomCommands", false );
             settings->hideWhenNotAvailable = interfaceGroup.readEntry( "HideWhenNotAvailable",false );
             settings->hideWhenNotExisting = interfaceGroup.readEntry( "HideWhenNotExisting", false );
             settings->activateStatistics = interfaceGroup.readEntry( "ActivateStatistics", false );
-            settings->trafficThreshold = interfaceGroup.readEntry( "TrafficThreshold", 0 );
+            settings->trafficThreshold = clamp<int>(interfaceGroup.readEntry( "TrafficThreshold", 0 ), 0, 1000 );
             if ( settings->customCommands )
             {
                 int numCommands = interfaceGroup.readEntry( "NumCommands", 0 );
@@ -239,69 +181,52 @@ void KNemoDaemon::reparseConfiguration()
                 }
             }
         }
-        settingsDict.insert( interface, settings );
+        settingsHash.insert( interface, settings );
     }
 
-    // load the settings for the plotter
-    KConfigGroup plotterGroup( config, "PlotterSettings" );
-    mPlotterSettings.pixel = plotterGroup.readEntry( "Pixel", 1 );
-    mPlotterSettings.count = plotterGroup.readEntry( "Count", 5 );
-    mPlotterSettings.distance = plotterGroup.readEntry( "Distance", 30 );
-    mPlotterSettings.fontSize = plotterGroup.readEntry( "FontSize", 8 );
-    mPlotterSettings.minimumValue = plotterGroup.readEntry( "MinimumValue", 0 );
-    mPlotterSettings.maximumValue = plotterGroup.readEntry( "MaximumValue", 1 );
-    mPlotterSettings.labels = plotterGroup.readEntry( "Labels", true );
-    mPlotterSettings.topBar = plotterGroup.readEntry( "TopBar", false );
-    mPlotterSettings.showIncoming = plotterGroup.readEntry( "ShowIncoming", true );
-    mPlotterSettings.showOutgoing = plotterGroup.readEntry( "ShowOutgoing", true );
-    mPlotterSettings.verticalLines = plotterGroup.readEntry( "VerticalLines", true );
-    mPlotterSettings.horizontalLines = plotterGroup.readEntry( "HorizontalLines", true );
-    mPlotterSettings.automaticDetection = plotterGroup.readEntry( "AutomaticDetection", true );
-    mPlotterSettings.verticalLinesScroll = plotterGroup.readEntry( "VerticalLinesScroll", true );
-    mPlotterSettings.colorVLines = plotterGroup.readEntry( "ColorVLines", mColorVLines.rgb() );
-    mPlotterSettings.colorHLines = plotterGroup.readEntry( "ColorHLines", mColorHLines.rgb() );
-    mPlotterSettings.colorIncoming = plotterGroup.readEntry( "ColorIncoming", mColorIncoming.rgb() );
-    mPlotterSettings.colorOutgoing = plotterGroup.readEntry( "ColorOutgoing", mColorOutgoing.rgb() );
-    mPlotterSettings.colorBackground = plotterGroup.readEntry( "ColorBackground", mColorBackground.rgb() );
-
-    // Remove all interfaces that the user deleted from
-    // the internal list.
-    foreach ( QString key, mInterfaceDict.keys() )
+    // Remove interfaces that are no longer monitored
+    foreach ( QString key, mInterfaceHash.keys() )
     {
-        if ( !settingsDict.contains( key ) )
+        if ( !settingsHash.contains( key ) )
         {
-            config->deleteGroup( "Interface_" + key );
-            Interface *interface = mInterfaceDict.take( key );
+            Interface *interface = mInterfaceHash.take( key );
             delete interface;
         }
     }
-    config->sync();
 
-    // Update all other interfaces and add new ones.
-    foreach ( QString key, settingsDict.keys() )
+    // Add/update those that do need to be monitored
+    foreach ( QString key, settingsHash.keys() )
     {
         Interface* iface;
-        if ( !mInterfaceDict.contains( key ) )
+        if ( !mInterfaceHash.contains( key ) )
         {
             iface = new Interface( key, mGeneralData, mPlotterSettings );
-            mInterfaceDict.insert( key, iface );
+            mInterfaceHash.insert( key, iface );
         }
         else
-            iface = mInterfaceDict.value( key );
+            iface = mInterfaceHash.value( key );
 
         InterfaceSettings& settings = iface->getSettings();
-        settings.alias = settingsDict.value( key )->alias;
-        settings.iconSet = settingsDict.value( key )->iconSet;
-        settings.customCommands = settingsDict.value( key )->customCommands;
-        settings.hideWhenNotAvailable = settingsDict.value( key )->hideWhenNotAvailable;
-        settings.hideWhenNotExisting = settingsDict.value( key )->hideWhenNotExisting;
-        settings.activateStatistics = settingsDict.value( key )->activateStatistics;
-        settings.trafficThreshold = settingsDict.value( key )->trafficThreshold;
-        settings.commands = settingsDict.value( key )->commands;
-        iface->configChanged();
+        settings.alias = settingsHash.value( key )->alias;
+        settings.iconSet = settingsHash.value( key )->iconSet;
+        settings.customCommands = settingsHash.value( key )->customCommands;
+        settings.hideWhenNotAvailable = settingsHash.value( key )->hideWhenNotAvailable;
+        settings.hideWhenNotExisting = settingsHash.value( key )->hideWhenNotExisting;
+        settings.activateStatistics = settingsHash.value( key )->activateStatistics;
+        settings.trafficThreshold = settingsHash.value( key )->trafficThreshold;
+        settings.commands = settingsHash.value( key )->commands;
+        iface->configChanged();  // important to activate the statistics
     }
 }
 
+void KNemoDaemon::reparseConfiguration()
+{
+    mPollTimer->stop();
+    readConfig();
+    // restart the timer with the new interval
+    mPollTimer->start( mGeneralData.pollInterval * 1000 );
+}
+
 QString KNemoDaemon::getSelectedInterface()
 {
     // Reset the variable to avoid preselecting an interface when
@@ -315,6 +240,27 @@ QString KNemoDaemon::getSelectedInterface()
 
 void KNemoDaemon::updateInterfaces()
 {
+    if ( !mBackend )
+        return;
+
+    if ( !mHaveInterfaces )
+    {
+        // If there's an interface for the default route, let's make that the
+        // one to watch
+        QString ifaceName = mBackend->getDefaultRouteIface();
+        if ( !ifaceName.isEmpty() )
+        {
+            Interface *iface = new Interface( ifaceName, mGeneralData, mPlotterSettings );
+            mInterfaceHash.insert( ifaceName, iface );
+
+            InterfaceSettings& settings = iface->getSettings();
+            settings.iconSet = "monitor";
+            settings.alias = ifaceName;
+            mHaveInterfaces = true;
+            iface->configChanged();
+        }
+    }
+
     mBackend->update();
 }
 
