@@ -83,7 +83,7 @@ KSignalPlotter::KSignalPlotter( QWidget *parent)
   mStackBeams = false;
   mFillOpacity = 20;
   mRescaleTime = 0;
-  mUnit = "KB/s";
+  mUnit = "KiB/s";
 }
 
 KSignalPlotter::~KSignalPlotter()
@@ -442,6 +442,13 @@ void KSignalPlotter::setShowAxis( bool value )
 #else
   mScrollableImage = QPixmap();
 #endif
+  if( mShowAxis ) {
+    if ( kapp->layoutDirection() == Qt::RightToLeft )
+      mPlottingArea.setRight(mPlottingArea.right() - mAxisTextWidth - 10);
+    else
+      mPlottingArea.setLeft(mAxisTextWidth+10);
+  }
+  update();
 }
 
 bool KSignalPlotter::showAxis() const
@@ -458,6 +465,7 @@ void KSignalPlotter::setAxisFont( const QFont &font )
 #else
   mScrollableImage = QPixmap();
 #endif
+  update();
 
 }
 
@@ -516,7 +524,14 @@ void KSignalPlotter::updateDataBuffers()
    *  +4 for extra data points so there is
    *     1) no wasted space and
    *     2) no loss of precision when drawing the first data point. */
-  mSamples = static_cast<uint>( ( ( width() - 2 ) /
+
+  /* Need to account for axis text width, or there may be a long delay before
+   * a rescale happens */
+  int axisWidth = 0;
+  if ( mShowAxis )
+    axisWidth = mAxisTextWidth + 10;
+
+  mSamples = static_cast<uint>( ( ( width() - axisWidth - 2 ) /
                                 mHorizontalScale ) + 4.5 );
 }
 
@@ -961,33 +976,28 @@ void KSignalPlotter::setFillOpacity(int fill)
 void KSignalPlotter::plotterAxisScaleChanged()
 {
     QString unit = mUnit.toUpper();
+    bool addSecs = unit.contains( "/S" );
     double value = maxValue();
-    if (unit == "KB" || unit  == "KIB")
+
+    if (value >= 1024*1024*0.7) //If it's over 0.7GiB, then set the scale to gigabytes
     {
-        if (value >= 1024*1024*0.7) //If it's over 0.7GiB, then set the scale to gigabytes
-        {
-            setScaleDownBy(1024*1024);
-            mUnit = "GiB";
-        }
-        else if (value > 1024)
-        {
-            setScaleDownBy(1024);
-            mUnit = "MiB";
-        }
+        setScaleDownBy(1024*1024);
+        mUnit = "GiB";
     }
-    else if (unit == "KB/S" || unit == "KIB/S")
+    else if (value > 1024)
     {
-        if (value >= 1024*1024*0.7)  //If it's over 0.7GiB, then set the scale to gigabytes
-        {
-            setScaleDownBy(1024*1024);
-            mUnit = "GiB/s";
-        }
-        else if (value > 1024)
-        {
-            setScaleDownBy(1024);
-            mUnit = "MiB/s";
-        }
+        setScaleDownBy(1024);
+        mUnit = "MiB";
     }
+    else
+    {
+        setScaleDownBy(1);
+        mUnit = "KiB";
+    }
+
+    if ( addSecs )
+        mUnit += "/s";
+
     setUnit(mUnit);
 }
 
